; Replication of the "Palette V.2" Cartridge
; More information here: http://www.atarimania.com/game-atari-2600-vcs-palette-v2_32971.html
; Charles Wieland, 1/18/2020
; junkosoft.com

	processor 6502
	include vcs.h

; Add constants
FRAMETODELAY = 15
BOXHEIGHT = 8

; Add Labels to RAM locations
SECTION = $80
STARTCOLOR = $81
BKCOLOR = $82
FRAMECOUNT = $83

	ORG $F000
Start
; Initialize the system
	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

ClearStack
	STA $00,X
	DEX
	BNE ClearStack

; Setup the static playfield pattern
	LDA #%11100000
	STA PF0
	LDA #%00111001
	STA PF1
	LDA #%01110011
	STA PF2

; Set the frame counter
	LDA #FRAMETODELAY
	STA FRAMECOUNT

; Start vertical sync, vertical blank
DrawVBlank
	LDA #$02
	STA VBLANK
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC
	LDA #$00
	STA VSYNC

; Set the timer for vertical blank (37 scanlines)
	LDA #$2C
	STA TIM64T

; Setup the number of sections to draw
	LDA #16
	STA SECTION
	LDA #00
	STA STARTCOLOR

; Set the background color
	LDA BKCOLOR
	STA COLUBK
	STA COLUPF

WaitForVBlankTimer
	LDA INTIM
	BNE WaitForVBlankTimer
	STA WSYNC
	STA VBLANK				;3

; Set the Y register to the number of scanlines to draw for each row of boxes
	LDY #BOXHEIGHT			;2

DrawSection:

; Define the playfield and draw 7 scanlines
; NOP values are placed to use clock cycles before changing the color
DrawLine:
	LDA STARTCOLOR			;4
	NOP						;2
	NOP						;2
	NOP						;2
	NOP						;2
	NOP						;2
	STA COLUPF				;3
	CLC						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	NOP						;2
	ADC #02					;3
	STA COLUPF				;3
	STA WSYNC				;3
	DEY						;2
	BNE DrawLine			;2/3

; Set the timer for five empty scanlines
	LDA #05
	STA TIM64T

; Set the playfield color to match the background color for the empty lines
	LDA BKCOLOR
	STA COLUPF
	LDX SECTION
	DEC SECTION
	LDY #BOXHEIGHT
	LDA STARTCOLOR
	CLC
	ADC #$10
	STA STARTCOLOR

WaitForEmptyLineTimer:
	LDA INTIM
	BNE WaitForEmptyLineTimer
	STA WSYNC

	; This is DEX instead of DEC SECTION in order to make the clock counts match for every scanline
	DEX						;2
	BNE DrawSection			;2/3

; Enable VBLANK before overscan
	LDA #$02
	STA VBLANK

; Set the timer for the overscan (30 scanlines)
	LDA #$24
	STA TIM64T

; Adjust the background color based upon the joystick input
	LDA #$10
	BIT SWCHA		; Up
	BNE CheckP0Down

	; Increment the frame count
	DEC FRAMECOUNT
	BNE CheckP0Down

	; Decrement the background color
	LDA BKCOLOR
	SEC
	SBC #$10
	STA BKCOLOR
	LDA #FRAMETODELAY
	STA FRAMECOUNT

CheckP0Down
	LDA #$20
	BIT SWCHA		; Up
	BNE CheckP0Left

	; Increment the frame count
	DEC FRAMECOUNT
	BNE CheckP0Left

	; Increment the background color
	LDA BKCOLOR
	CLC
	ADC #$10
	STA BKCOLOR
	LDA #FRAMETODELAY
	STA FRAMECOUNT

CheckP0Left
	LDA #$40
	BIT SWCHA		; Left
	BNE CheckP0Right

	; Increment the frame count
	DEC FRAMECOUNT
	BNE CheckP0Right

	; Decrement the background color
	LDA BKCOLOR
	SEC
	SBC #$02
	STA BKCOLOR
	LDA #FRAMETODELAY
	STA FRAMECOUNT

CheckP0Right:	
	LDA #$80
	BIT SWCHA		; Right
	BNE WaitForOverscanTimer

	; Decrement the frame count
	DEC FRAMECOUNT
	BNE WaitForOverscanTimer

	; Increment the background color
	LDA BKCOLOR
	CLC
	ADC #$02
	STA BKCOLOR
	LDA #FRAMETODELAY
	STA FRAMECOUNT

WaitForOverscanTimer:
	LDA INTIM
	BNE WaitForOverscanTimer
	STA WSYNC

	LDA #$00
	STA VBLANK
	JMP DrawVBlank

	org $F7FC
	.word Start
	.word Start